var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { html, LitElement } from 'lit';
import { customElement, property } from 'lit/decorators.js';
const defaultEmptyValue = '';
function isValueEmpty(value) {
    return value === '' || value === null || value === undefined;
}
function isValidEmptyValueAttribute(attribute) {
    return ['', 'null', 'undefined'].includes(attribute);
}
function emptyValueAttributeToProperty(attribute) {
    if (attribute === 'null') {
        return null;
    }
    if (attribute === 'undefined') {
        return undefined;
    }
    return '';
}
let TextInput = class TextInput extends LitElement {
    constructor() {
        super(...arguments);
        this.value = defaultEmptyValue;
        this.emptyValue = defaultEmptyValue;
    }
    attributeChangedCallback(name, oldValue, newValue) {
        super.attributeChangedCallback(name, oldValue, newValue);
        switch (name) {
            case 'value':
                this.value = newValue || this.emptyValue;
                break;
            case 'empty-value':
                if (isValidEmptyValueAttribute(newValue)) {
                    this.emptyValue = emptyValueAttributeToProperty(newValue);
                }
                else {
                    console.warn(`Invalid empty-value "${newValue}". Please use '', 'null', or 'undefined'.`);
                }
                break;
        }
    }
    willUpdate(changedProperties) {
        if (changedProperties.has('emptyValue') && isValueEmpty(this.value)) {
            this.value = this.emptyValue;
        }
    }
    handleInput(e) {
        this.value = e.target.value || this.emptyValue;
    }
    render() {
        if (isValueEmpty(this.value) && this.value !== this.emptyValue) {
            console.warn(`value is empty but does not match emptyValue. value: "${this.value}", emptyValue: "${this.emptyValue}"`);
        }
        return html `<input
      .value=${this.value ?? ''}
      @input=${this.handleInput}
    />`;
    }
};
__decorate([
    property()
], TextInput.prototype, "value", void 0);
__decorate([
    property({ attribute: 'empty-value' })
], TextInput.prototype, "emptyValue", void 0);
TextInput = __decorate([
    customElement('text-input-lit')
], TextInput);
export { TextInput };
//# sourceMappingURL=text-input.js.map